@@ -65,7 +65,7 @@ mca_pml_ob1_t mca_pml_ob1 = {
         mca_pml_ob1_start,
         mca_pml_ob1_dump,
         mca_pml_ob1_ft_event,
-        32768,
+        65535,
         INT_MAX
     }
 };
@@ -183,11 +183,19 @@ int mca_pml_ob1_add_comm(ompi_communicator_t* comm)
     if (NULL == pml_comm) {
         return OMPI_ERR_OUT_OF_RESOURCE;
     }
+
+    /* should never happen, but it was, so check */
+    if (comm->c_contextid > (uint32_t) mca_pml_ob1.super.pml_max_contextid) {
+        OBJ_RELEASE(pml_comm);
+        return OMPI_ERR_OUT_OF_RESOURCE;
+    }
+
     mca_pml_ob1_comm_init_size(pml_comm, comm->c_remote_group->grp_proc_count);
     comm->c_pml_comm = pml_comm;
 
     for( i = 0; i < comm->c_remote_group->grp_proc_count; i++ ) {
         pml_comm->procs[i].ompi_proc = ompi_group_peer_lookup(comm->c_remote_group,i);
+        OBJ_RETAIN(pml_comm->procs[i].ompi_proc);
     }
     /* Grab all related messages from the non_existing_communicator pending queue */
     for( item = opal_list_get_first(&mca_pml_ob1.non_existing_communicator_pending);
@@ -413,9 +421,15 @@ static void mca_pml_ob1_fin_completion( mca_btl_base_module_t* btl,
     MCA_PML_OB1_PROGRESS_PENDING(bml_btl);
 }
 
+/**
+ * Send an FIN to the peer. If we fail to send this ack (no more available
+ * fragments or the send failed) this function automatically add the FIN
+ * to the list of pending FIN, Which guarantee that the FIN will be sent
+ * later.
+ */
 int mca_pml_ob1_send_fin( ompi_proc_t* proc,
                           mca_bml_base_btl_t* bml_btl,
-                          void *hdr_des,
+                          ompi_ptr_t hdr_des,
                           uint8_t order,
                           uint32_t status )
 {
@@ -438,7 +452,7 @@ int mca_pml_ob1_send_fin( ompi_proc_t* proc,
     hdr = (mca_pml_ob1_fin_hdr_t*)fin->des_src->seg_addr.pval;
     hdr->hdr_common.hdr_flags = 0;
     hdr->hdr_common.hdr_type = MCA_PML_OB1_HDR_TYPE_FIN;
-    hdr->hdr_des.pval = hdr_des;
+    hdr->hdr_des = hdr_des;
     hdr->hdr_fail = status;
 
     ob1_hdr_hton(hdr, MCA_PML_OB1_HDR_TYPE_FIN, proc);
@@ -481,7 +495,7 @@ void mca_pml_ob1_process_pending_packets(mca_bml_base_btl_t* bml_btl)
         if(NULL == send_dst) {
             OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
             opal_list_append(&mca_pml_ob1.pckt_pending,
-                    (opal_list_item_t*)pckt);
+                             (opal_list_item_t*)pckt);
             OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
             continue;
         }
@@ -494,29 +508,30 @@ void mca_pml_ob1_process_pending_packets(mca_bml_base_btl_t* bml_btl)
                         pckt->hdr.hdr_ack.hdr_dst_req.pval,
                         pckt->hdr.hdr_ack.hdr_send_offset,
                         pckt->hdr.hdr_common.hdr_flags & MCA_PML_OB1_HDR_FLAGS_NORDMA);
-                MCA_PML_OB1_PCKT_PENDING_RETURN(pckt);
-                if(OMPI_ERR_OUT_OF_RESOURCE == rc) {
-                    MCA_PML_OB1_ADD_ACK_TO_PENDING(pckt->proc,
-                            pckt->hdr.hdr_ack.hdr_src_req.lval,
-                            pckt->hdr.hdr_ack.hdr_dst_req.pval,
-                            pckt->hdr.hdr_ack.hdr_send_offset);
+                if( OPAL_UNLIKELY(OMPI_ERR_OUT_OF_RESOURCE == rc) ) {
+                    OPAL_THREAD_LOCK(&mca_pml_ob1.lock);
+                    opal_list_append(&mca_pml_ob1.pckt_pending,
+                                     (opal_list_item_t*)pckt);
+                    OPAL_THREAD_UNLOCK(&mca_pml_ob1.lock);
                     return;
                 }
                 break;
             case MCA_PML_OB1_HDR_TYPE_FIN:
                 rc = mca_pml_ob1_send_fin(pckt->proc, send_dst,
-                                          pckt->hdr.hdr_fin.hdr_des.pval,
+                                          pckt->hdr.hdr_fin.hdr_des,
                                           pckt->order,
                                           pckt->hdr.hdr_fin.hdr_fail);
-                MCA_PML_OB1_PCKT_PENDING_RETURN(pckt);
-                if(OMPI_ERR_OUT_OF_RESOURCE == rc)
-                     return;
+                if( OPAL_UNLIKELY(OMPI_ERR_OUT_OF_RESOURCE == rc) ) {
+                    return;
+                }
                 break;
             default:
                 opal_output(0, "[%s:%d] wrong header type\n",
-                        __FILE__, __LINE__);
+                            __FILE__, __LINE__);
                 break;
         }
+        /* We're done with this packet, return it back to the free list */
+        MCA_PML_OB1_PCKT_PENDING_RETURN(pckt);
     }
 }
 
