@@ -241,6 +241,7 @@ void orte_plm_base_launch_failed(orte_jobid_t job, pid_t pid,
 {
     orte_job_t *jdata;
     char *pidstr;
+    int sts;
     
     if (!opal_atomic_trylock(&orte_abort_inprogress_lock)) { /* returns 1 if already locked */
         OPAL_OUTPUT_VERBOSE((1, orte_plm_globals.output,
@@ -251,10 +252,17 @@ void orte_plm_base_launch_failed(orte_jobid_t job, pid_t pid,
     }
     
     OPAL_OUTPUT_VERBOSE((5, orte_plm_globals.output,
-                         "%s plm:base:launch_failed for job %s",
+                         "%s plm:base:launch_failed for job %s, status %d",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
-                         ORTE_JOBID_PRINT(job)));
+                         ORTE_JOBID_PRINT(job), status));
 
+    /* no matter what, we must exit with a non-zero status */
+    if (0 == status) {
+        sts = ORTE_ERROR_DEFAULT_EXIT_CODE;
+    } else {
+        sts = status;
+    }
+    
     /* if this is the daemon job that failed, set the flag indicating
      * that a daemon failed so we use the proper
      * methods for attempting to shutdown the rest of the system
@@ -277,17 +285,21 @@ void orte_plm_base_launch_failed(orte_jobid_t job, pid_t pid,
             if (WCOREDUMP(status)) {
                 orte_show_help("help-plm-base.txt", "daemon-died-signal-core", true,
                                pidstr, WTERMSIG(status));
+                sts = WTERMSIG(status);
             } else {
                 orte_show_help("help-plm-base.txt", "daemon-died-signal", true,
                                pidstr, WTERMSIG(status));
+                sts = WTERMSIG(status);
             }
 #else
             orte_show_help("help-plm-base.txt", "daemon-died-signal", true,
                             pidstr, WTERMSIG(status));
+            sts = WTERMSIG(status);
 #endif /* WCOREDUMP */
         } else {
             orte_show_help("help-plm-base.txt", "daemon-died-no-signal", true,
                            pidstr, WEXITSTATUS(status));
+            sts = WEXITSTATUS(status);
         }
         orted_failed_launch = true;
         free(pidstr);
@@ -308,7 +320,7 @@ void orte_plm_base_launch_failed(orte_jobid_t job, pid_t pid,
     
 WAKEUP:
     /* set orterun's exit code and wakeup so it can exit */
-    ORTE_UPDATE_EXIT_STATUS(status);
+    ORTE_UPDATE_EXIT_STATUS(sts);
     orte_trigger_event(&orte_exit);
 }
 
@@ -480,6 +492,20 @@ int orte_plm_base_daemon_callback(orte_std_cntr_t num_daemons)
  * for their local procs.
  */
 static bool app_launch_failed;
+static orte_vpid_t num_daemons_reported=0;
+static opal_event_t *dmn_report_ev=NULL;
+
+/* catch timeout to allow cmds to progress */
+static void timer_cb(int fd, short event, void *cbdata)
+{
+    /* free event */
+    if (NULL != dmn_report_ev) {
+        free(dmn_report_ev);
+        dmn_report_ev = NULL;
+    }
+    /* declare time is up */
+    app_launch_failed = true;
+}
 
 /* since the HNP also reports launch of procs, we need to separate out
  * the processing of the message vs its receipt so that the HNP
@@ -505,6 +531,13 @@ void orte_plm_base_app_report_launch(int fd, short event, void *data)
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
                          ORTE_NAME_PRINT(&mev->sender)));
     
+    /* got a response - cancel the timer */
+    if (NULL != dmn_report_ev) {
+        opal_event_del(dmn_report_ev);
+        free(dmn_report_ev);
+        dmn_report_ev = NULL;
+    }
+    
     /* unpack the jobid being reported */
     cnt = 1;
     if (ORTE_SUCCESS != (rc = opal_dss.unpack(buffer, &jobid, &cnt, ORTE_JOBID))) {
@@ -524,6 +557,8 @@ void orte_plm_base_app_report_launch(int fd, short event, void *data)
         goto CLEANUP;
     }
     
+    num_daemons_reported++;
+
     /* get the job data object */
     if (NULL == (jdata = orte_get_job_data_object(jobid))) {
         ORTE_ERROR_LOG(ORTE_ERR_NOT_FOUND);
@@ -605,6 +640,10 @@ void orte_plm_base_app_report_launch(int fd, short event, void *data)
                 jdata->aborted_proc = procs[vpid];  /* only store this once */
                 jdata->state = ORTE_JOB_STATE_FAILED_TO_START; /* update the job state */
             }
+            /* ensure we have a non-zero exit code */
+            if (0 == jdata->aborted_proc->exit_code) {
+                jdata->aborted_proc->exit_code = ORTE_ERROR_DEFAULT_EXIT_CODE;
+            }
             app_launch_failed = true;
             goto CLEANUP;
         }
@@ -616,6 +655,14 @@ void orte_plm_base_app_report_launch(int fd, short event, void *data)
         ORTE_ERROR_LOG(rc);
     }
     
+    if (orte_report_launch_progress) {
+        if (0 == num_daemons_reported % 100 || num_daemons_reported == orte_process_info.num_procs) {
+            opal_output(orte_clean_output, "Reported: %d (out of %d) daemons -  %d (out of %d) procs",
+                        (int)num_daemons_reported, (int)orte_process_info.num_procs,
+                        (int)jdata->num_launched, (int)jdata->num_procs);
+        }
+    }
+
     OPAL_OUTPUT_VERBOSE((5, orte_plm_globals.output,
                          "%s plm:base:app_report_launch completed processing",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME)));
@@ -624,11 +671,17 @@ CLEANUP:
     if (app_launch_failed) {
         if (NULL == jdata) {
             orte_errmgr.incomplete_start(ORTE_JOBID_INVALID, ORTE_ERROR_DEFAULT_EXIT_CODE);
+        } else if (NULL == jdata->aborted_proc) {
+            orte_errmgr.incomplete_start(jdata->jobid, ORTE_ERROR_DEFAULT_EXIT_CODE);
         } else {
             orte_errmgr.incomplete_start(jdata->jobid, jdata->aborted_proc->exit_code);
         }
     }
-    
+
+    /* restart the timer, if necessary */
+    if (jdata->num_launched < jdata->num_procs && 0 < orte_startup_timeout) {
+        ORTE_DETECT_TIMEOUT(&dmn_report_ev, orte_startup_timeout, 1000, 10000000, timer_cb);
+    }
 }
 
 
@@ -667,7 +720,7 @@ static int orte_plm_base_report_launched(orte_jobid_t job)
 {
     int rc;
     orte_job_t *jdata;
-
+    
     OPAL_OUTPUT_VERBOSE((5, orte_plm_globals.output,
                          "%s plm:base:report_launched for job %s",
                          ORTE_NAME_PRINT(ORTE_PROC_MY_NAME),
@@ -679,6 +732,13 @@ static int orte_plm_base_report_launched(orte_jobid_t job)
         return ORTE_ERR_NOT_FOUND;
     }
     
+    /* setup a timer - if we don't hear back from a daemon in the
+     * defined time, then we know things have failed
+     */
+    if (0 < orte_startup_timeout) {
+        ORTE_DETECT_TIMEOUT(&dmn_report_ev, orte_startup_timeout, 1000, 10000000, timer_cb);
+    }
+
     /* we should get a callback from every daemon that is involved in
      * the launch. Fortunately, the mapper keeps track of this number
      * for us since num_nodes = num_participating_daemons
@@ -861,6 +921,18 @@ int orte_plm_base_orted_append_basic_args(int *argc, char ***argv,
         }
     }
 
+    /* if output-filename was specified, pass that along */
+    if (NULL != orte_output_filename) {
+        opal_argv_append(argc, argv, "--output-filename");
+        opal_argv_append(argc, argv, orte_output_filename);
+    }
+    
+    /* if --xterm was specified, pass that along */
+    if (NULL != orte_xterm) {
+        opal_argv_append(argc, argv, "--xterm");
+        opal_argv_append(argc, argv, orte_xterm);
+    }
+    
     /* 
      * Pass along the Aggregate MCA Parameter Sets
      */
