@@ -10,6 +10,7 @@
  * Copyright (c) 2004-2005 The Regents of the University of California.
  *                         All rights reserved.
  * Copyright (c) 2006      Cisco Systems, Inc.  All rights reserved.
+ * Copyright (c) 2009      IBM Corporation.  All rights reserved.
  * $COPYRIGHT$
  * 
  * Additional copyrights may follow
@@ -77,6 +78,11 @@ static int allocate(opal_list_t *nodes)
         return ORTE_ERR_NOT_FOUND;
     }
     
+    /* save that value in the global job ident string for
+     * later use in any error reporting
+     */
+    orte_job_ident = strdup(pbs_jobid);
+    
     if (ORTE_SUCCESS != (ret = discover(nodes, pbs_jobid))) {
         ORTE_ERROR_LOG(ret);
         return ret;
